const morgan = require('morgan');
const rfs = require('rotating-file-stream');

class Log {
  register(server) {
    const {
      LOG_FOLDER,
      ACCESS_LOG_FILE_NAME,
      ACCESS_LOG_FORMAT,
      ACCESS_LOG_MAX_FILES,
      ACCESS_LOG_ROTATE_INTERVAL,
    } = process.env;
    if (!LOG_FOLDER || LOG_FOLDER.includes('<>')) return;
    const isEmpty = (value) => ( value || '<>' ).includes('<>');
    const isInvalidNumber = (value) => Number.isNaN(Number(value));
    const logFormat = isEmpty(ACCESS_LOG_FORMAT) ? '[:date] [:method] [:url] [:status] [:response-time ms]' : ACCESS_LOG_FORMAT;
    const interval = isEmpty(ACCESS_LOG_ROTATE_INTERVAL) ? '1d' : ACCESS_LOG_ROTATE_INTERVAL;
    const maxFiles = isEmpty(ACCESS_LOG_MAX_FILES) || isInvalidNumber(ACCESS_LOG_MAX_FILES) ? 90 : Number(ACCESS_LOG_MAX_FILES);
    const logFileName = isEmpty(ACCESS_LOG_FILE_NAME) ? 'access.log' : ACCESS_LOG_FILE_NAME;
    const accessLogStream = rfs.createStream(logFileName, {
      interval,
      path: LOG_FOLDER,
      maxFiles,
      intervalBoundary: true,
      initialRotation: true,
    });
    server.use(morgan(logFormat, { stream: accessLogStream }));
  }
}

module.exports = new Log();
