const DB = require('../database/DB');

class AssistantConfig {
  UPDATE = `
    UPDATE ASSISTANT_CONFIG
      SET NOME      = :NOME,
          DESCRICAO = :DESCRICAO,
          TIPO      = :TIPO,
          ORDEM     = :ORDEM,
          ATIVO     = :ATIVO,
          COMANDO   = :COMANDO,
          INTERVALO = :INTERVALO,
          COMPLEMENTOTIPO = :COMPLEMENTOTIPO,
          DTULTATU  = SYSDATE
    WHERE ID = :ID
  `;

  INSERT = `
    INSERT INTO ASSISTANT_CONFIG
      (NOME, DESCRICAO, TIPO, ORDEM, COMANDO, INTERVALO, COMPLEMENTOTIPO, IDMONITORAMENTO, ATIVO, DTINCLUSAO, DTULTATU)
    VALUES
      (:NOME, :DESCRICAO, :TIPO, :ORDEM, :COMANDO, :INTERVALO, :COMPLEMENTOTIPO, :IDMONITORAMENTO, :ATIVO, SYSDATE, SYSDATE)
  `;

  DELETE = `
    DELETE FROM ASSISTANT_CONFIG
    WHERE
      INSTR(:ID, '_' || ID || '_') <> 0
  `;

  GET_ASSISTANT_CONFIG = `
    SELECT
      ID, NOME, DESCRICAO, TIPO, ORDEM, ATIVO, COMANDO, INTERVALO, COMPLEMENTOTIPO, IDMONITORAMENTO, DTULTATU
    FROM ASSISTANT_CONFIG
    ORDER BY ORDEM, NOME
  `;

  GET_ACTIVE_ASSISTANT_CONFIG = `
    SELECT
      ID, NOME, DESCRICAO, TIPO, ORDEM, ATIVO, COMANDO, INTERVALO, COMPLEMENTOTIPO, IDMONITORAMENTO, DTULTATU
    FROM ASSISTANT_CONFIG
    WHERE
      ATIVO = 'S'
    ORDER BY ORDEM, NOME
  `;

  GET_MONITORING_PARAMS = `
    SELECT
      ID,
      NOME,
      TIPO,
      COMANDO,
      INTERVALO,
      COMPLEMENTOTIPO,
      IDMONITORAMENTO
    FROM
      ASSISTANT_CONFIG
    WHERE
      ATIVO = 'S'
  `;

  GET_MONITORING_PARAMS_BY_TYPE = `
    SELECT
      ID,
      NOME,
      TIPO,
      COMANDO,
      INTERVALO,
      COMPLEMENTOTIPO
    FROM
      ASSISTANT_CONFIG
    WHERE
      ATIVO = 'S' AND
      TIPO = :TIPO
  `;
  insert(params, executionParams = {}) {
    const execParams = Object.assign({}, {
      outParams: [{name: 'ID', type: 'number'}],
    }, executionParams);
    return DB.execute(this.INSERT, params, execParams);
  }
  update(params, executionParams = {}) {
    return DB.execute(this.UPDATE, params, executionParams);
  }
  delete(params, executionParams = {}) {
    return DB.execute(this.DELETE, params, executionParams);
  }
  getAssistantConfig() {
    return DB.fetch(this.GET_ASSISTANT_CONFIG);
  }
  getActiveAssistantConfig() {
    return DB.fetch(this.GET_ACTIVE_ASSISTANT_CONFIG);
  }
  getMonitoringParams() {
    return DB.fetch(this.GET_MONITORING_PARAMS);
  }
  getMonitoringParamsByType(params) {
    return DB.fetch(this.GET_MONITORING_PARAMS_BY_TYPE, params);
  }
}

module.exports = new AssistantConfig();