const DB = require('../database/DB');

class UpdaterEventosCx {
  INSERT = `
    INSERT INTO
    UPDATEREVENTOSCX (
      CDFILIAL,
      CDCAIXA,
      CDEVENTO,
      CDUUID,
      CDPRODUCTID,
      CDPRODFILIID,
      IDSTEVENTO,
      NRVERSAOATUAL,
      NRVERSAOALVO,
      IDCRIARVERSAO,
      DTINCLUSAO,
      DTULTATU
    ) VALUES (
      :P_CDFILIAL,
      :P_CDCAIXA,
      :P_CDEVENTO,
      :P_CDUUID,
      :P_CDPRODUCTID,
      :P_CDPRODFILIID,
      :P_IDSTEVENTO,
      :P_NRVERSAOATUAL,
      :P_NRVERSAOALVO,
      :P_IDCRIARVERSAO,
      SYSDATE,
      SYSDATE
    )
  `;

  DELETE_PRODUCT_EVENTS = `
    DELETE FROM
      UPDATEREVENTOSCX
    WHERE
      CDFILIAL    = :CDFILIAL     AND
      CDCAIXA     = :CDCAIXA      AND
      CDPRODUCTID = :CDPRODUCTID
  `;

  UPDATE_EVENT_STATUS = `
    UPDATE UPDATEREVENTOSCX
    SET
      IDSTEVENTO = :P_IDSTEVENTO,
      DTULTATU   = SYSDATE
    WHERE
      CDEVENTO   = :P_CDEVENTO
  `;

  CANCEL_EVENT = `
    UPDATE
      UPDATEREVENTOSCX
    SET
      IDSTEVENTO = 'CANCELADO',
      DTULTATU = SYSDATE
    WHERE
      CDFILIAL = :P_CDFILIAL AND
      (
        (:P_CDCAIXA IS NULL) OR
        (CDCAIXA = :P_CDCAIXA)
      )
  `;

  GET_PRODUCT_EVENTS = `
    SELECT
      *
    FROM
      UPDATEREVENTOSCX
    WHERE
      CDFILIAL    = :P_CDFILIAL    AND
      CDCAIXA     = :P_CDCAIXA     AND
      IDSTEVENTO  = :P_IDSTEVENTO
  `;

  GET_ACTIVE_EVENTS = `
    SELECT
      *
    FROM
      UPDATEREVENTOSCX
    WHERE
      CDFILIAL = :P_CDFILIAL AND
      CDCAIXA  = :P_CDCAIXA  AND
      IDSTEVENTO NOT IN ('CANCELADO', 'FINALIZADO')
  `;

  GET_EVENTS_IN_PROGRESS = `
    SELECT
      *
    FROM
      UPDATEREVENTOSCX
    WHERE
      CDFILIAL = :P_CDFILIAL AND
      INSTR(:P_CDCAIXA, '_' || CDCAIXA || '_') <> 0 AND
      IDSTEVENTO IN (
        'INSTALACAO_EM_CURSO',
        'REMOCAO_EM_CURSO',
        'REMOCAO_PENDENTE',
        'VERIFICACAO_EM_CURSO',
        'ANALISE_INTEGRIDADE_EM_CURSO',
        'ANALISE_INTEGRIDADE_PENDENTE'
      )
  `;

  GET_PENDING_EVENTS = `
    SELECT
      UE.CDFILIAL,
      UE.CDCAIXA,
      UPH.CDOPERADOR,
      UE.CDEVENTO,
      UE.CDPRODUCTID,
      UE.CDPRODFILIID,
      UE.IDSTEVENTO,
      UE.NRVERSAOATUAL,
      UE.NRVERSAOALVO,
      UE.IDCRIARVERSAO,
      UE.DTINCLUSAO,
      UPV.DSDIRETORIO,
      UPV.IDDOWNLOADVER,
      UPC.CONFIG
    FROM
      UPDATEREVENTOSCX UE
      LEFT JOIN UPDATERPRODVERSAO UPV ON (
        UE.CDPRODUCTID  = UPV.UPDATERPRODID AND
        UE.NRVERSAOALVO = UPV.NRVERSAO
      )
      LEFT JOIN UPDATERPRODCONFIGCX UPC ON (
        UE.CDPRODUCTID  = UPC.CDPRODUCTID AND
        UE.NRVERSAOALVO = UPC.NRVERSAO AND UE.CDCAIXA = UPC.CDCAIXA
      )
      LEFT JOIN UPDATERHISTORICOCX UPH ON (
        UE.CDEVENTO = UPH.CDEVENTO
      )
    WHERE
      UE.CDFILIAL = :P_CDFILIAL AND
      UE.CDCAIXA  = :P_CDCAIXA  AND
      UE.IDSTEVENTO IN ('INSTALACAO_PENDENTE', 'INSTALACAO_EM_CURSO')
    UNION
    SELECT
      UE.CDFILIAL,
      UE.CDCAIXA,
      UPH.CDOPERADOR,
      UE.CDEVENTO,
      UE.CDPRODUCTID,
      UE.CDPRODFILIID,
      UE.IDSTEVENTO,
      UE.NRVERSAOATUAL,
      UE.NRVERSAOALVO,
      UE.IDCRIARVERSAO,
      UE.DTINCLUSAO,
      UPV.DSDIRETORIO,
      UPV.IDDOWNLOADVER,
      '' AS CONFIG
    FROM
      UPDATEREVENTOSCX UE
      LEFT JOIN UPDATERPRODVERSAO UPV ON (
        UE.CDPRODUCTID   = UPV.UPDATERPRODID AND
        UE.NRVERSAOATUAL = UPV.NRVERSAO
      )
      LEFT JOIN UPDATERHISTORICOCX UPH ON (
        UE.CDEVENTO = UPH.CDEVENTO
      )
    WHERE
      UE.CDFILIAL = :P_CDFILIAL AND
      UE.CDCAIXA  = :P_CDCAIXA  AND
      UE.IDSTEVENTO IN ('REMOCAO_PENDENTE', 'REMOCAO_EM_CURSO')
  `;

  insert(params, executionParams = {}) {
    return DB.execute(this.INSERT, params, executionParams);
  }

  deleteProductEvents(params, executionParams = {}) {
    return DB.execute(this.DELETE_PRODUCT_EVENTS, params, executionParams);
  }

  updateEventStatus(params, executionParams = {}) {
    return DB.execute(this.UPDATE_EVENT_STATUS, params, executionParams);
  }

  cancelEvent(params, executionParams = {}) {
    return DB.execute(this.CANCEL_EVENT, params, executionParams);
  }

  getProductEvents(params) {
    return DB.fetch(this.GET_PRODUCT_EVENTS, params);
  }

  getActiveEvents(params) {
    return DB.fetch(this.GET_ACTIVE_EVENTS, params);
  }

  getEventsInProgress(params) {
    return DB.fetch(this.GET_EVENTS_IN_PROGRESS, params);
  }

  getPendingEvents(params) {
    return DB.fetch(this.GET_PENDING_EVENTS, params);
  }
}

module.exports = new UpdaterEventosCx();