const DateHandler = require("../functions/DateHandler");
const VersionCheck = require("../functions/VersionCheck");
const SystemLogger = require("../logs/SystemLogger");
const updaterModel = require("../model/updaterModel");
const Emojis = require("../templates/Emojis");

class UpdaterTelegramData {
  async getUserAndProductName(CDOPERADOR, DEVOPSID) {
    const { rows } = await updaterModel.getOperatorAndProductName({
      CDOPERADOR,
      DEVOPSID,
    });
    const { 
      NMOPERADOR,
      NMPRODUCT
    } = rows.length ? rows.shift() : {};
    return {
      user: NMOPERADOR || `Nome do operador ${CDOPERADOR} não encontrado`,
      product: NMPRODUCT || `Nome do produto ${DEVOPSID} não encontrado`,
    };
  }

  async getMultiInstallData(commonData) {
    const {
      UNITS,
      DEVOPSID,
      CDOPERADOR,
    } = commonData;
    commonData.productId = DEVOPSID;
    const userAndProduct = await this.getUserAndProductName(CDOPERADOR, DEVOPSID);
    Object.assign(commonData, userAndProduct);
    commonData.action = 'Instalação Múltipla';
    commonData.unitId = UNITS.map((unit) => unit.CDFILIAL).join(', ');
    return commonData;
  }

  async getSingleInstallData(commonData) {
    const {
      IDSTEVENTO,
      INSTALLVERSION,
      CURRENTVERSION,
      DEVOPSID,
      CDOPERADOR,
    } = commonData;
    const userAndProduct = await this.getUserAndProductName(CDOPERADOR, DEVOPSID);
    Object.assign(commonData, userAndProduct);
    commonData.productId = DEVOPSID;
    commonData.action = 'Remoção';

    if (IDSTEVENTO === 'INSTALACAO_PENDENTE') {
      commonData.action = 'Nova Instalação';
      if (CURRENTVERSION) {
        const isUpdate = VersionCheck.greaterThan(INSTALLVERSION, CURRENTVERSION);
        commonData.action = isUpdate ? 'Atualização' : 'Desatualização';
        commonData.icon = isUpdate ? Emojis.ARROW_UP : Emojis.ARROW_DOWN;
        const versionChange = `Versão: ${CURRENTVERSION} -> ${INSTALLVERSION}`;
        commonData.upgradeInfo = `${commonData.icon} ${versionChange}\n`;
      }
    }
    if(commonData.CDFILIAL) {
      commonData.unitId = commonData.CDFILIAL;
    }
    return commonData;
  }

  getSelfUpdaterData(commonData) {
    const data = {
      product: 'Assistant Updater',
      action: 'Atualização',
      productId: '0000',
      unitId: commonData.CDFILIAL || '',
      ...commonData,
    };
    return data;
  }

  getCommonData(initialData) {
    const commonData = {
      date: DateHandler.toString({ format: 'DD/MM/YYYY HH:mm:ss' }),
      ...initialData,
    };
    if (initialData.NRORG) {
      commonData.orgId = initialData.NRORG;
    }
    return commonData;
  }

  async getData(initialData, type) {
    const commonData = this.getCommonData(initialData);
    switch(type) {
      case 'single-install': return this.getSingleInstallData(commonData);
      case 'multi-install': return this.getMultiInstallData(commonData);
      case 'self-updater': return this.getSelfUpdaterData(commonData);
    }
    SystemLogger.write(`INVALID_TYPE: {${type}}`);
    return {};
  }
}

module.exports = new UpdaterTelegramData();